<?php

class DbEntity_Table
{
	private $DB = null;
	private $Tables = array();
	private $Fields = array();
	private $FieldPKeys = array();
	private $FieldsInfo = array();
	private $Rows = array();

	public function __construct($DB)
	{
		$this->DB = $DB;
	}
	
	public function GetNextID($IdFieldName = "ID")
	{
		$ID = 0;
		try
		{
			$sql = "select max(" . $IdFieldName . ") as ID from " . $this->Entity_Row_Table;
			$Res = $this->DB->query($sql);
			if ($Res->num_rows == 1)
			{
				$Row = $Res->fetch_assoc();
				$ID = $Row["ID"] + 1;
				if ($ID == null)
					$ID = 1;
			}
		} catch (Exception $e)
		{
			echo 'ERROR: ', $e->getMessage(), "\n";
		}
		return $ID;
	}

	public function GetLastAutoIncID()
	{
		return $this->DB->insert_id;
	}
	
	// CREATE A NEW VOID ROW IN THE MEMORY
	public function CreateVoidRow($Table=null)
	{
		if ($Table==null){$Table = $this->Tables[0];}
		
		$Sql = "select * from " . $Table . " limit 1";
		$result = $this->DB->Query($Sql);
		if (!$result)
		{
			echo("SQL ERROR: " . $this->DB->error . "<br/>");
			return null;
		}
		else
		{
			$this->LoadTableInfo($result);
			$Rows = array();

			$Row = new DbEntity_Row($this->DB, $this->FieldPKeys, $this->FieldsInfo);
			foreach ($this->FieldsInfo as $Info)
			{
				$Row->SetValue($Info->orgname, null);
			}
			array_push($this->Rows, $Row);
			return $Row;
		}
	}

	// RETURN A LIST OF FIELD INFORMATION
	public function GetFieldsInfo($Table=null)
	{
		if (isset($Table))
		{
			$this->Table = $Table;
			$Qry = "select * from " . $Table ." limit 1";
			$result = $DB->Query($Sql);
			if (!$result)
			{
				echo("SQL ERROR: " . $DB->error . "<br/>");
			}
			else
			{
				LoadTableInfo($result);
			}
		}
		return $this->FieldsInfo;
	}

	private function LoadTableInfo($result)
	{
		try
		{
			// GET INFO
			$K = 0;
			
			for ($N = 0; $N < $result->field_count; $N++)
			{
				$Info = $result->fetch_field_direct($N);
				if (!in_array($Info->table,$this->Tables,)) {array_push($this->Tables,$Info->table);}
				if (($Info->charsetnr == 33) && (in_array($Info->type, $this->Type_Str)))
				{
					$Info->length /= 3;
				}
				if (($Info->flags & 2) == 2)
				{
					$this->FieldPKeys[$K] = $Info->orgname;
					$K++;
				}
				if ($Info->name != "")
				{
					$this->FieldsInfo[$Info->name] = $Info;
				}
				else
				{
					$this->FieldsInfo[$Info->oriname] = $Info;
				}
			}
		} catch (Exception $e)
		{
			echo 'ERROR: ', $e->getMessage(), "\n";
		}
	}
	
	// CARICA I DATI DI UNA TABELLA
	public function Query($SqlQuery)
	{
		try
		{
			$this->Table = "";
			$this->Rows = array();
			$result = $this->DB->query($SqlQuery);
			if (!$result)
			{
				echo("SQL ERROR: " . $this->DB->error . "<br/>" . $SqlQuery . "<br/>");
			}
			else
			{
				$this->LoadTableInfo($result);
				for ($R = 0; $R < $result->num_rows; $R++)
				{
					$Row = new DbEntity_Row($this->DB, $this->FieldPKeys, $this->FieldsInfo, true);
					$DataRow = $result->fetch_assoc();
					foreach ($this->FieldsInfo as $Info)
					{
						if (!in_array($Info->table, $this->Tables))
							{array_push($this->Tables, $Info->table);}
							
						$Row->SetValue($Info->name, $DataRow[$Info->name]);
					}
					array_push($this->Rows, $Row);
				}
			}
		} catch (Exception $e)
		{
			echo 'ERROR: ', $e->getMessage(), "\n";
		}
	}

	// CARICA I DATI DI UNA TABELLA
	public function QueryTable($Table, $Where = "", $OrderBy="", $Limit="")
	{
		try
		{
			$this->Table = $Table;
			$Sql = "select * from " . $Table;
			if ($Where != "")
			{
				$Sql .= " where " . $Where;
			}

			if ($OrderBy != "")
			{
				$Sql .= " order by " . $OrderBy;
			}
			
			if ($Limit != "")
			{
				$Sql .= " limit " . $Limit;
			}
			
			$result = $this->DB->query($Sql);
			if (!$result)
			{
				echo("SQL ERROR: " . $this->DB->error . "<br/>" . $Sql . "<br/>");
			}
			else
			{
				$this->LoadTableInfo($result);
				$this->Rows = array();
				for ($R = 0; $R < $result->num_rows; $R++)
				{
					$Row = new DbEntity_Row($this->DB, $this->FieldPKeys, $this->FieldsInfo, true);
					$DataRow = $result->fetch_assoc();
					foreach ($this->FieldsInfo as $Info)
					{
						$Row->SetValue($Info->name, $DataRow[$Info->name]);
					}
					array_push($this->Rows, $Row);
				}
			}
		} catch (Exception $e)
		{
			echo 'ERROR: ', $e->getMessage(), "\n";
			echo 'ERROR: ', $Sql, "\n";
			exit(0);
		}
	}

	// RETURN THE NUBER OF LOADED ROWS
	public function GetRowCount()
	{
		return count($this->Rows);
	}

	// RETURN A DbEntity_Row OBJECT
	public function GetRow($Position)
	{
		if ((count($this->Rows)-1)<$Position) return null;
		return $this->Rows[$Position];
	}

	// GET THE LIST OF ALL DbEntity_Row OBJECTS
	public function GetAllRows()
	{
		return $this->Rows;
	}

	// RETURN THE LIST OF THE TABLE PKEY
	public function GetPKeyList()
	{
		return $this->FieldPKeys;
	}

	// RETURN THE LIST OF THE FIELDS INFO
	public function GetFielsInfo($FieldName)
	{
		return $this->FieldsInfo[$FieldName];
	}

	public function GetTables()
	{
		return $this->Tables;
	}
	
	// WRITE THE CHANGE TO THE DB
	public function Update()
	{
		foreach ($this->GetAllRows() as $R)
		{
			if ($R->IsModified())
			{
				$R->Update();
			}
		}
	}
}

class DbEntity_Row
{
	// FIELD TYPES
	private static $Type_Str = array(252, 253, 254);
	private static $Type_Num = array(1, 2, 3, 4, 5, 8, 9, 16, 246);
	private static $Type_DateTime = array(12,7);
	private static $Type_Date = array(10);
	private $Entity_Row_DB = null;
	private $Entity_Row_KeyWherePart = "";
	private $Entity_Row_RowExists = false;
	private $Entity_Row_Modified = false;
	private $Entity_Row_Tables = array();
	private $Entity_Row_FieldPKeys = array();
	private $Entity_Row_FieldsInfo = array();
	
	public function __construct($DB, $FieldPKeys, $FieldsInfo, $RowExist = false)
	{
		foreach ($FieldsInfo as $Info)
		{
			if (!in_array($Info->table, $this->Entity_Row_Tables))
				{array_push($this->Entity_Row_Tables, $Info->table);}
		}
		$this->Entity_Row_DB = $DB;
		$this->Entity_Row_FieldPKeys = $FieldPKeys;
		$this->Entity_Row_FieldsInfo = $FieldsInfo;
		$this->Entity_Row_RowExist = $RowExist;
	} 

	// CHECK IF THE ROW WILL BE CHANGED BY USER
	public function IsModified()
	{
		return $this->Entity_Row_Modified;
	}
	
	// UPDATE THE THE ROW
	public function Update($Debug=false)
	{
		if ($this->Entity_Row_KeyWherePart == "")
		{
			$this->BuildKeyWherePart();
		}
		if (!$this->Entity_Row_RowExists)
		{
			$this->CheckRowExistOnDB();
		}
		
		foreach ($this->Entity_Row_Tables as $Table)
		{
			if ($this->Entity_Row_RowExists)
			{
				// UPDATE
				$FldList = "";
				$WherePart = "";
				foreach ($this->Entity_Row_FieldsInfo as $Fld)
				{
					if ($Fld->table == $Table)
					{
						if ($FldList != "")
							{$FldList .= ",";}
						else
							{$WherePart=$this->BuildKeyWherePart($Table);}
							
						$Val = $this->FormatFiledValue($Fld, $this->{$Fld->orgname});
						$FldList .= $Fld->orgname . "=" . $Val;
					}
				}
				$sql = "update " . $Table . " set " . $FldList . " where " . $WherePart;
			}
			else
			{
				// INSERT
				$FldList = "";
				foreach ($this->Entity_Row_FieldsInfo as $Fld)
				{
					if ($Fld->table == $Table)
					{
						$Val = $this->{$Fld->orgname};
						if (isset($Val))
						{
							if ($FldList != "")
							{
								$FldList .= ",";
							}
							$FldList .= "`" . $Fld->orgname . "`";
						}
					}
				}

				$FldVal = "";
				foreach ($this->Entity_Row_FieldsInfo as $Fld)
				{
					if ($Fld->table == $Table)
					{
						$Val = $this->{$Fld->orgname};
						if (isset($Val))
						{
							if ($FldVal != "")
							{
								$FldVal .= ",";
							}
							$FldVal .= $this->FormatFiledValue($Fld, $Val);
						}
					}
				}
				$sql = "insert into " . $Table . " (" . $FldList . ") values (" . $FldVal . ")";
				$this->Entity_Row_RowExists = true;
			}

			try
			{
				if ($Debug) {echo $sql; file_put_contents ("DbEntity.log",$sql, FILE_APPEND);}
				$this->Entity_Row_DB->query($sql);
				$this->Entity_Row_Modified = false;
			} catch (Exception $e)
			{
				echo "FUNCTION: Update\n";
				echo 'ERROR: ', $e->getMessage(), $sql , "\n";
				exit(0);
			}
		}
	}

	public function CheckRowExistOnDB($Table=null,$WherePart=null)
	{
		if (!isset($Table)){$Table=$this->Entity_Row_Tables[0];}
		if (!isset($WherePart)){$WherePart=$this->Entity_Row_KeyWherePart;}
		try
		{
			$Sql = "select * from " . $Table . " where " . $WherePart;

			$result = $this->Entity_Row_DB->Query($Sql);
			if (!$result)
			{
				echo("SQL ERROR: " . $this->Entity_Row_DB->error ."<br>". $Sql. "<br/>");
				return null;
			}
			else
			{
				if ($result->num_rows > 0)
				{
					$this->Entity_Row_RowExists = true;
					return true;
				}
				else
				{
					$this->Entity_Row_RowExists = false;
					return false;
				}
			}
		} catch (Exception $e)
		{
			echo "FUNCTION: CheckRowExistOnDB\n";
			echo "ERROR: ", $e->getMessage(), "\n";
			exit(0);
		}
		return null;
	}
	
	// GET A THE VALUE OF A FIELD
	public function GetValue($FieldName)
	{
		return $this->{$FieldName};
	}

	// GET A THE STRI§NG OF A FIELD
	public function GetString($FieldName)
	{
		if ($this->{$FieldName} == null)
		{
			return "";
		}
		else
		{
			return $this->{$FieldName};
		}
	}

	// SET THE VALUE IN A ROW FIELD
	public function SetValue($FieldName, $Value)
	{
		$this->Entity_Row_Modified = true;
		$this->{$FieldName} = $Value;
	}

	// GET THE LIST OF THE PRIMARY KEYS OF THE TABLE
	public function GetPKeyList()
	{
		return $this->Entity_Row_FieldPKeys;
	}

	public function GetTables($FieldName=null)
	{
		return $this->Entity_Row_Tables;
	}
	
	// RETURN THE TABLE NAME
	public function GetTableName($FieldName=null)
	{
		if(isset($FieldName))
		{
			if (!isset($this->Entity_Row_FieldsInfo[$FieldName])) return "";
			return $this->Entity_Row_FieldsInfo[$FieldName]->table;
		}
		else
		{
			if (!isset($this->Entity_Row_FieldsInfo[0])) return "";
			return $this->Entity_Row_FieldsInfo[0]->table;
		}
	}

	// GET THE INFO OF A FIELD
	public function GetFielsInfo($FieldName)
	{
		if (!isset($this->Entity_Row_FieldsInfo[$FieldName])) return null;
		return $this->Entity_Row_FieldsInfo[$FieldName];
	}

	// FORMAT A VALUE FOR A FIELD TIPE
	public function FormatFiledValue($Info, $Value)
	{
		try
		{
			$Ret = $Value;
			if (in_array($Info->type, DbEntity_Row::$Type_Str))
			{
				$Ret="'".$this->Entity_Row_DB->real_escape_string($Value)."'";
			}

			if (in_array($Info->type, DbEntity_Row::$Type_Num))
			{
				if (is_numeric($Value))
				{
					$Ret = $Value;
				}
				else
				{
					$Ret = "0";
				}
			}

			if (in_array($Info->type, DbEntity_Row::$Type_DateTime))
			{
				$Ret = "'" . $Value . "'";
			}

			if (in_array($Info->type, DbEntity_Row::$Type_Date))
			{
				$Ret = "'" . $Value . "'";
			}
		} catch (Exception $e)
		{
			echo "FUNCTION: FormatFiledValue\n";
			echo "ERROR: ", $e->getMessage(), "\n";
		}
		return $Ret;
	}

	// BUILD A WHERE PART STRING WITH THE PKEY OF THE ROW
	public function BuildKeyWherePart($Table=null)
	{
		$Ret = "";
		if (!isset($Table)){$Table=$this->Entity_Row_Tables[0];}
		try
		{
			foreach ($this->Entity_Row_FieldPKeys as $PKey)
			{
				if ($this->Entity_Row_FieldsInfo[$PKey]->table == $Table)
				{
					if ($Ret != "")
					{
						$Ret .= " and ";
					}
					$Val = $this->FormatFiledValue($this->Entity_Row_FieldsInfo[$PKey], $this->{$PKey});
					$Ret .= $PKey . "=" . $Val;
				}
			}
			$this->Entity_Row_KeyWherePart = $Ret;
			return $Ret;

		} catch (Exception $e)
		{
			echo "FUNCTION: BuildKeyWherePart\n";
			echo "ERROR: ", $e->getMessage(), "\n";
		}
	}
}
